//! Package Manager for Aios
//! 
//! Simple package management system inspired by Redox pkgutils
//! Provides basic package operations: install, remove, list, update

#![allow(dead_code)]
use core::fmt;
use crate::filesystem::{SimpleFS, FileType};

/// Package metadata structure
#[derive(Debug, Clone, Copy)]
pub struct Package {
    pub name: [u8; 32],    // Fixed-size name
    pub version: [u8; 16], // Fixed-size version
    pub description: [u8; 64], // Fixed-size description
    pub status: PackageStatus,
    pub size: usize,
}

/// Package status
#[derive(Debug, Clone, Copy, PartialEq)]
pub enum PackageStatus {
    Installed,
    Available,
    Broken,
}

/// Package manager error types
#[derive(Debug)]
pub enum PackageError {
    NotFound,
    AlreadyInstalled,
    NotInstalled,
    InvalidName,
    IoError,
}

impl fmt::Display for PackageError {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            PackageError::NotFound => write!(f, "Package not found"),
            PackageError::AlreadyInstalled => write!(f, "Package already installed"),
            PackageError::NotInstalled => write!(f, "Package not installed"),
            PackageError::InvalidName => write!(f, "Invalid package name"),
            PackageError::IoError => write!(f, "I/O error"),
        }
    }
}

/// Package Manager
pub struct PackageManager {
    installed_packages: [Option<Package>; 32], // Simple fixed-size array
    package_count: usize,
    filesystem: SimpleFS,
}

impl PackageManager {
    /// Create new package manager
    pub fn new() -> Self {
        PackageManager {
            installed_packages: [None; 32],
            package_count: 0,
            filesystem: SimpleFS::new(),
        }
    }

    /// Install a package
    pub fn install(&mut self, name: &str) -> Result<(), PackageError> {
        // Check if package name is valid
        if name.is_empty() || name.len() > 32 {
            return Err(PackageError::InvalidName);
        }

        // Check if already installed
        if self.is_installed(name) {
            return Err(PackageError::AlreadyInstalled);
        }

        // Check if we have space
        if self.package_count >= 32 {
            return Err(PackageError::IoError);
        }

        // Create package from predefined list
        let package = self.create_package(name)?;
        
        // Install package files to filesystem
        self.install_package_files(name)?;
        
        // Add to installed packages
        self.installed_packages[self.package_count] = Some(package);
        self.package_count += 1;

        Ok(())
    }

    /// Remove a package
    pub fn remove(&mut self, name: &str) -> Result<(), PackageError> {
        // Find package
        let mut found_index = None;
        for (i, pkg) in self.installed_packages.iter().enumerate() {
            if let Some(package) = pkg {
                if self.str_eq(&package.name, name) {
                    found_index = Some(i);
                    break;
                }
            }
        }

        match found_index {
            Some(index) => {
                // Remove package files from filesystem
                let _ = self.remove_package_files(name);
                
                // Remove package by shifting array
                for i in index..self.package_count.saturating_sub(1) {
                    self.installed_packages[i] = self.installed_packages[i + 1].take();
                }
                self.package_count -= 1;
                Ok(())
            }
            None => Err(PackageError::NotInstalled),
        }
    }

    /// List installed packages
    pub fn list(&self) -> &[Option<Package>] {
        &self.installed_packages[..self.package_count]
    }

    /// Check if package is installed
    pub fn is_installed(&self, name: &str) -> bool {
        self.installed_packages[..self.package_count]
            .iter()
            .any(|pkg| {
                if let Some(package) = pkg {
                    self.str_eq(&package.name, name)
                } else {
                    false
                }
            })
    }

    /// Get package count
    pub fn count(&self) -> usize {
        self.package_count
    }

    /// Helper function to compare string with byte array
    fn str_eq(&self, arr: &[u8; 32], s: &str) -> bool {
        let s_bytes = s.as_bytes();
        if s_bytes.len() > 32 {
            return false;
        }
        
        // Find the end of the string in the array (null terminator)
        let mut arr_len = 0;
        for i in 0..32 {
            if arr[i] == 0 {
                break;
            }
            arr_len += 1;
        }
        
        if arr_len != s_bytes.len() {
            return false;
        }
        
        for i in 0..arr_len {
            if arr[i] != s_bytes[i] {
                return false;
            }
        }
        
        true
    }
    
    /// Copy string to byte array
    fn copy_str_to_array<const N: usize>(&self, s: &str) -> [u8; N] {
        let mut arr = [0u8; N];
        let bytes = s.as_bytes();
        let len = core::cmp::min(bytes.len(), N - 1); // Leave space for null terminator
        
        for i in 0..len {
            arr[i] = bytes[i];
        }
        
        arr
    }

    /// Create a package from predefined list
    fn create_package(&self, name: &str) -> Result<Package, PackageError> {
        match name {
            "vim" => Ok(Package {
                name: self.copy_str_to_array("vim"),
                version: self.copy_str_to_array("8.2.0"),
                description: self.copy_str_to_array("Vi IMproved text editor"),
                status: PackageStatus::Installed,
                size: 1024,
            }),
            "git" => Ok(Package {
                name: self.copy_str_to_array("git"),
                version: self.copy_str_to_array("2.39.0"),
                description: self.copy_str_to_array("Git version control system"),
                status: PackageStatus::Installed,
                size: 2048,
            }),
            "curl" => Ok(Package {
                name: self.copy_str_to_array("curl"),
                version: self.copy_str_to_array("7.88.0"),
                description: self.copy_str_to_array("Command line tool for transferring data"),
                status: PackageStatus::Installed,
                size: 512,
            }),
            "rust" => Ok(Package {
                name: self.copy_str_to_array("rust"),
                version: self.copy_str_to_array("1.75.0"),
                description: self.copy_str_to_array("Rust programming language"),
                status: PackageStatus::Installed,
                size: 4096,
            }),
            "python" => Ok(Package {
                name: self.copy_str_to_array("python"),
                version: self.copy_str_to_array("3.11.0"),
                description: self.copy_str_to_array("Python programming language"),
                status: PackageStatus::Installed,
                size: 3072,
            }),
            "claude" => Ok(Package {
                name: self.copy_str_to_array("claude"),
                version: self.copy_str_to_array("1.0.0"),
                description: self.copy_str_to_array("Claude AI integration service"),
                status: PackageStatus::Installed,
                size: 1536,
            }),
            _ => Err(PackageError::NotFound),
        }
    }

    /// Install package files to filesystem
    fn install_package_files(&mut self, name: &str) -> Result<(), PackageError> {
        match name {
            "vim" => {
                // Create vim executable
                let vim_content = b"#!/bin/sh\necho 'Vi IMproved - text editor'\necho 'Usage: vim [file]'\n";
                self.filesystem.create_file("/bin/vim", vim_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
                
                // Create vim config
                let vimrc_content = b"\" Vim configuration\nset number\nset autoindent\nsyntax on\n";
                self.filesystem.create_file("/etc/vimrc", vimrc_content, FileType::Regular)
                    .map_err(|_| PackageError::IoError)?;
            }
            "git" => {
                let git_content = b"#!/bin/sh\necho 'Git - version control system'\necho 'Usage: git [command]'\n";
                self.filesystem.create_file("/bin/git", git_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
                
                let gitconfig_content = b"[core]\n    editor = vim\n[user]\n    name = Aios User\n";
                self.filesystem.create_file("/etc/gitconfig", gitconfig_content, FileType::Regular)
                    .map_err(|_| PackageError::IoError)?;
            }
            "curl" => {
                let curl_content = b"#!/bin/sh\necho 'cURL - data transfer tool'\necho 'Usage: curl [URL]'\n";
                self.filesystem.create_file("/bin/curl", curl_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
            }
            "rust" => {
                let rustc_content = b"#!/bin/sh\necho 'Rust Compiler'\necho 'Usage: rustc [file.rs]'\n";
                self.filesystem.create_file("/bin/rustc", rustc_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
                
                let cargo_content = b"#!/bin/sh\necho 'Cargo - Rust package manager'\necho 'Usage: cargo [command]'\n";
                self.filesystem.create_file("/bin/cargo", cargo_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
            }
            "python" => {
                let python_content = b"#!/bin/sh\necho 'Python interpreter'\necho 'Usage: python [script.py]'\n";
                self.filesystem.create_file("/bin/python", python_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
                
                let pip_content = b"#!/bin/sh\necho 'pip - Python package installer'\necho 'Usage: pip install [package]'\n";
                self.filesystem.create_file("/bin/pip", pip_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
            }
            "claude" => {
                let claude_content = b"#!/bin/sh\necho 'Claude AI Assistant'\necho 'Usage: claude [query]'\n";
                self.filesystem.create_file("/bin/claude", claude_content, FileType::Executable)
                    .map_err(|_| PackageError::IoError)?;
                
                let config_content = b"# Claude AI Configuration\napi_endpoint=https://api.anthropic.com\nmodel=claude-3-sonnet\n";
                self.filesystem.create_file("/etc/claude.conf", config_content, FileType::Regular)
                    .map_err(|_| PackageError::IoError)?;
            }
            _ => return Err(PackageError::NotFound),
        }
        Ok(())
    }

    /// Remove package files from filesystem
    fn remove_package_files(&mut self, name: &str) -> Result<(), PackageError> {
        match name {
            "vim" => {
                let _ = self.filesystem.delete_file("/bin/vim");
                let _ = self.filesystem.delete_file("/etc/vimrc");
            }
            "git" => {
                let _ = self.filesystem.delete_file("/bin/git");
                let _ = self.filesystem.delete_file("/etc/gitconfig");
            }
            "curl" => {
                let _ = self.filesystem.delete_file("/bin/curl");
            }
            "rust" => {
                let _ = self.filesystem.delete_file("/bin/rustc");
                let _ = self.filesystem.delete_file("/bin/cargo");
            }
            "python" => {
                let _ = self.filesystem.delete_file("/bin/python");
                let _ = self.filesystem.delete_file("/bin/pip");
            }
            "claude" => {
                let _ = self.filesystem.delete_file("/bin/claude");
                let _ = self.filesystem.delete_file("/etc/claude.conf");
            }
            _ => return Err(PackageError::NotFound),
        }
        Ok(())
    }

    /// List installed files
    pub fn list_files(&self) -> &[Option<crate::filesystem::FileEntry>] {
        self.filesystem.list_files()
    }

    /// Get filesystem reference
    pub fn filesystem(&self) -> &SimpleFS {
        &self.filesystem
    }
}

impl Package {
    /// Get package name as string slice
    pub fn name_str(&self) -> &str {
        // Find null terminator
        let mut len = 0;
        for i in 0..32 {
            if self.name[i] == 0 {
                break;
            }
            len += 1;
        }
        
        // Safe to unwrap since we control the input
        core::str::from_utf8(&self.name[..len]).unwrap_or("")
    }
    
    /// Get package version as string slice  
    pub fn version_str(&self) -> &str {
        let mut len = 0;
        for i in 0..16 {
            if self.version[i] == 0 {
                break;
            }
            len += 1;
        }
        
        core::str::from_utf8(&self.version[..len]).unwrap_or("")
    }
}

impl fmt::Display for PackageStatus {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            PackageStatus::Installed => write!(f, "installed"),
            PackageStatus::Available => write!(f, "available"),
            PackageStatus::Broken => write!(f, "broken"),
        }
    }
}