//! Lightweight Package Manager for Aios
//! 
//! Memory-safe package management system with minimal memory footprint

#![allow(dead_code)]

/// Lightweight package entry
#[derive(Debug, Clone, Copy)]
pub struct LightPackage {
    pub name: [u8; 16],    // Reduced from 32 to 16 bytes
    pub version: [u8; 8],  // Reduced from 16 to 8 bytes
    pub status: PackageStatus,
}

/// Package status
#[derive(Debug, Clone, Copy, PartialEq)]
pub enum PackageStatus {
    Installed,
    Available,
}

/// Lightweight Package Manager
pub struct LightPackageManager {
    packages: [Option<LightPackage>; 8], // Reduced from 32 to 8 packages
    count: usize,
}

impl LightPackageManager {
    pub fn new() -> Self {
        LightPackageManager {
            packages: [None; 8],
            count: 0,
        }
    }

    /// Install a package (lightweight - no actual files)
    pub fn install(&mut self, name: &str) -> Result<(), &'static str> {
        if name.len() > 15 {
            return Err("Package name too long");
        }

        if self.count >= 8 {
            return Err("Package manager full");
        }

        // Check if already installed
        if self.is_installed(name) {
            return Err("Already installed");
        }

        // Create package entry
        let package = self.create_package(name)?;
        self.packages[self.count] = Some(package);
        self.count += 1;

        Ok(())
    }

    /// Remove a package
    pub fn remove(&mut self, name: &str) -> Result<(), &'static str> {
        let mut found_index = None;
        for (i, pkg) in self.packages.iter().enumerate() {
            if let Some(package) = pkg {
                if self.str_eq(&package.name, name) {
                    found_index = Some(i);
                    break;
                }
            }
        }

        match found_index {
            Some(index) => {
                // Remove package by shifting array
                for i in index..self.count.saturating_sub(1) {
                    self.packages[i] = self.packages[i + 1];
                }
                self.packages[self.count - 1] = None;
                self.count -= 1;
                Ok(())
            }
            None => Err("Package not installed"),
        }
    }

    /// List installed packages
    pub fn list(&self) -> &[Option<LightPackage>] {
        &self.packages[..self.count]
    }

    /// Check if package is installed
    pub fn is_installed(&self, name: &str) -> bool {
        self.packages[..self.count]
            .iter()
            .any(|pkg| {
                if let Some(package) = pkg {
                    self.str_eq(&package.name, name)
                } else {
                    false
                }
            })
    }

    /// Get package count
    pub fn count(&self) -> usize {
        self.count
    }

    /// Create a package from predefined list
    fn create_package(&self, name: &str) -> Result<LightPackage, &'static str> {
        match name {
            "vim" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("vim"),
                version: self.copy_str_to_array::<8>("8.2"),
                status: PackageStatus::Installed,
            }),
            "git" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("git"),
                version: self.copy_str_to_array::<8>("2.39"),
                status: PackageStatus::Installed,
            }),
            "curl" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("curl"),
                version: self.copy_str_to_array::<8>("7.88"),
                status: PackageStatus::Installed,
            }),
            "rust" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("rust"),
                version: self.copy_str_to_array::<8>("1.75"),
                status: PackageStatus::Installed,
            }),
            "python" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("python"),
                version: self.copy_str_to_array::<8>("3.11"),
                status: PackageStatus::Installed,
            }),
            "claude" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("claude"),
                version: self.copy_str_to_array::<8>("1.0"),
                status: PackageStatus::Installed,
            }),
            "zsh" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("zsh"),
                version: self.copy_str_to_array::<8>("5.9"),
                status: PackageStatus::Installed,
            }),
            "bash" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("bash"),
                version: self.copy_str_to_array::<8>("5.2"),
                status: PackageStatus::Installed,
            }),
            "fish" => Ok(LightPackage {
                name: self.copy_str_to_array::<16>("fish"),
                version: self.copy_str_to_array::<8>("3.6"),
                status: PackageStatus::Installed,
            }),
            _ => Err("Package not found"),
        }
    }

    /// Helper function to compare string with byte array
    fn str_eq(&self, arr: &[u8; 16], s: &str) -> bool {
        let s_bytes = s.as_bytes();
        if s_bytes.len() > 16 {
            return false;
        }
        
        // Find the end of the string in the array (null terminator)
        let mut arr_len = 0;
        for i in 0..16 {
            if arr[i] == 0 {
                break;
            }
            arr_len += 1;
        }
        
        if arr_len != s_bytes.len() {
            return false;
        }
        
        for i in 0..arr_len {
            if arr[i] != s_bytes[i] {
                return false;
            }
        }
        
        true
    }
    
    /// Copy string to byte array
    fn copy_str_to_array<const N: usize>(&self, s: &str) -> [u8; N] {
        let mut arr = [0u8; N];
        let bytes = s.as_bytes();
        let len = core::cmp::min(bytes.len(), N - 1); // Leave space for null terminator
        
        for i in 0..len {
            arr[i] = bytes[i];
        }
        
        arr
    }
}

impl LightPackage {
    /// Get package name as string slice
    pub fn name_str(&self) -> &str {
        // Find null terminator
        let mut len = 0;
        for i in 0..16 {
            if self.name[i] == 0 {
                break;
            }
            len += 1;
        }
        
        // Safe to unwrap since we control the input
        core::str::from_utf8(&self.name[..len]).unwrap_or("")
    }
    
    /// Get package version as string slice  
    pub fn version_str(&self) -> &str {
        let mut len = 0;
        for i in 0..8 {
            if self.version[i] == 0 {
                break;
            }
            len += 1;
        }
        
        core::str::from_utf8(&self.version[..len]).unwrap_or("")
    }
}